@@ -1,7 +1,7 @@
 # ---------- ChainSetup-----------
 
 package Audio::Nama::ChainSetup;
-use Audio::Nama::Globals qw($file $config $jack $setup $this_engine %tn %bn $mode :trackrw);
+use Audio::Nama::Globals qw($file $config $jack $setup %tn %bn %en $mode :trackrw $this_engine);
 use Audio::Nama::Log qw(logsub logpkg);
 use Modern::Perl;
 use Data::Dumper::Concise;
@@ -46,9 +46,8 @@ our (
 
 sub remove_temporary_tracks {
 	logsub("&remove_temporary_tracks");
-	map { logpkg(__FILE__,__LINE__,'debug',"removing temporary track ",$_->name); $_->remove  } 
-		grep{ $_->group eq 'Temp'} 
-		Audio::Nama::audio_tracks();
+	map { logpkg(__FILE__,__LINE__,'debug',"removing temporary track ",$_->group.'/'.$_->name); $_->remove  } 
+		grep{ $_->group eq 'Temp' } Audio::Nama::audio_tracks();
 }
 sub initialize {
 
@@ -64,6 +63,7 @@ sub initialize {
 	Audio::Nama::disable_length_timer();
 	reset_aux_chain_counter();
 	unlink $file->chain_setup;
+	delete $this_engine->{valid_setup};
 	$g;
 }
 sub ecasound_chain_setup { $chain_setup } 
@@ -83,7 +83,7 @@ sub is_engine_track {
 	$Audio::Nama::ti{$n} if $is_ecasound_chain{$n}
 }
 sub engine_wav_out_tracks {
-	grep{$_->rec_status eq REC} engine_tracks();
+	grep{$_->rec} engine_tracks();
 }
 # return file output entries, including Mixdown 
 sub really_recording { 
@@ -101,10 +101,6 @@ sub generate_setup_try {
 
 	my $extra_setup_code = shift;
 
-	# in an ideal CS world, all of the following routing
-	# routines (add_paths_for_*) would be accomplished by
-	# the track or bus itself, rather than handcoded below.
-	
 	# start with bus routing
 	
 	map{ $_->apply($g) } Audio::Nama::Bus::all();
@@ -116,8 +112,8 @@ sub generate_setup_try {
 	add_paths_for_aux_sends();
 	logpkg(__FILE__,__LINE__,'debug',"Graph after aux sends:\n$g");
 
-	add_paths_from_Master();
-	logpkg(__FILE__,__LINE__,'debug',"Graph with paths from Master:\n$g");
+	add_paths_from_Main();
+	logpkg(__FILE__,__LINE__,'debug',"Graph with paths from Main:\n$g");
 
 	add_paths_for_mixdown_handling();
 	logpkg(__FILE__,__LINE__,'debug',"Graph with mixdown mods:\n$g");
@@ -138,12 +134,12 @@ sub generate_setup_try {
 
 	logpkg(__FILE__,__LINE__,'debug',"Graph with inserts:\n$g");
 
-	# Mix tracks to mono if Master is mono
+	# Mix tracks to mono if Main is mono
 	# (instead of just throwing away right channel)
 
-	if ($g->has_vertex('Master') and $tn{Master}->width == 1)
+	if ($g->has_vertex('Main') and $tn{Main}->width == 1)
 	{
-		$g->set_vertex_attribute('Master', 'ecs_extra' => '-chmix:1')
+		$g->set_vertex_attribute('Main', 'ecs_extra' => '-chmix:1')
 	}
 	#logpkg(__FILE__,__LINE__,'info',sub{"Graph object dump:\n",Dumper($g)});
 
@@ -153,48 +149,54 @@ sub generate_setup_try {
 		write_chains(); 
 		1
 	} else { 
-		Audio::Nama::throw("No tracks to record or play.");
+		Audio::Nama::throw("No audio tracks to record or play.");
 		0
 	}
 }
 
-sub add_paths_for_aux_sends {
+sub add_paths_for_aux_sends { # not including Main
 
 	# currently this routing is track-oriented 
 
 	# we could add this to the Audio::Nama::Bus base class
-	# then suppress it in Mixdown and Master groups
+	# then suppress it in Mixdown and Main groups
 
 	logsub("&add_paths_for_aux_sends");
 
 	map {  Audio::Nama::Graph::add_path_for_aux_send($g, $_ ) } 
 	grep { (ref $_) !~ /Slave/ 
-			and $_->group !~ /Mixdown|Master/
+			and $_->group !~ /Mixdown|Null/
 			and $_->send_type 
 			and $_->rec_status ne OFF } Audio::Nama::audio_tracks();
 }
 
 
-sub add_paths_from_Master {
-	logsub("&add_paths_from_Master");
+sub add_paths_from_Main {
+	logsub("&add_paths_from_Main");
 
 	if ($mode->mastering){
-		$g->add_path(qw[Master Eq Low Boost]);
+		$g->add_path(qw[Main Eq Low Boost]);
 		$g->add_path(qw[Eq Mid Boost]);
 		$g->add_path(qw[Eq High Boost]);
 	}
-	my $final_leg_origin = $mode->mastering ?  'Boost' : 'Master';
-	$g->add_path($final_leg_origin, output_node($tn{Master}->send_type)) 
-		if $tn{Master}->rw ne OFF
+	else { 
+		$g->add_path('Main', output_node($tn{Main}->send_type)) if $tn{Main}->mon
+			and ! $tn{Mixdown}->rec
+
+		# tests require this, why not generated by
+		# add_paths_for_aux_sends() ??
+	}
 
 }
 sub add_paths_for_mixdown_handling {
 	logsub("&add_paths_for_mixdown_handling");
+	my $final_leg_origin = $mode->mastering ? 'Boost' : 'Main';
 
-	if ($tn{Mixdown}->rec_status eq REC){
+	if ($tn{Mixdown}->rw eq REC ){
 		# don't monitor via soundcard
-		$g->delete_edge('Master','soundcard_out');
-		my @p = (($mode->mastering ? 'Boost' : 'Master'), ,'Mixdown', 'wav_out');
+		$g->delete_edge('Main','soundcard_out');
+		$g->delete_edge('Boost','soundcard_out');
+		my @p = ($final_leg_origin, ,'Mixdown', 'wav_out');
 		$g->add_path(@p);
 		$g->set_vertex_attributes('Mixdown', {
 		  	format_template		=> $config->{mix_to_disk_format},
@@ -204,12 +206,12 @@ sub add_paths_for_mixdown_handling {
 												 
 	# Mixdown handling - playback
 	
-	} elsif ($tn{Mixdown}->rec_status eq PLAY){ 
-			my @e = ('wav_in','Mixdown',output_node($tn{Master}->send_type));
+	} elsif ($tn{Mixdown}->rw eq PLAY and $tn{Mixdown}->playback_version()){ 
+			my @e = ('wav_in','Mixdown',output_node($tn{Main}->send_type));
 			$g->add_path(@e);
 			$g->set_vertex_attributes('Mixdown', {
-				send_type	=> $tn{Master}->send_type,
-				send_id		=> $tn{Master}->send_id,
+				send_type	=> $tn{Main}->send_type,
+				send_id		=> $tn{Main}->send_id,
 				chain			=> "Mixdown" }); 
 		# no effects will be applied because effects are on chain 2
 	}
@@ -242,21 +244,19 @@ sub process_routing_graph {
 	map { 
 		$inputs{$_->ecs_string} //= [];
 		push @{$inputs{$_->ecs_string}}, $_->chain_id;
-		# post-input modifiers
-		$post_input{$_->chain_id} = $_->ecs_extra if $_->ecs_extra;
+		$post_input{$_->chain_id} .= $_->ecs_extra if $_->ecs_extra;
+		$post_input{$_->chain_id} .= join ' ', map{ $_->ecasound_format } $_->channel_ops if $_->channel_ops 
 	} 
 	grep { $_->direction eq 'input' } @io;
 
-	# sort chain_ids by output
-
 	map { 
 		$outputs{$_->ecs_string} //= [];
 		push @{$outputs{$_->ecs_string}}, $_->chain_id;
-		# pre-output modifers
 		$pre_output{$_->chain_id} = $_->ecs_extra if $_->ecs_extra;
 	} 
 	grep { $_->direction eq 'output' } @io;
 
+
 	no warnings 'numeric';
 	my @in_keys = values %inputs;
 	my @out_keys = values %outputs;
@@ -295,7 +295,7 @@ sub non_track_dispatch {
 	# assign chain_id to edge based on chain_id of left-side loop's
 	# corresponding track:
 	#	
-	# hihat_out -- J7a -> Master_in
+	# hihat_out -- J7a -> Main_in
 	#
 	# soundcard_in -> wav_out (rec_file)
 	#
@@ -369,7 +369,7 @@ sub dispatch { # creates an IO object from a graph edge
 	my $track = $tn{$name};
 	my $class = Audio::Nama::IO::get_class( $endpoint, $direction );
 		# we need the $direction because there can be 
-		# edges to and from loop,Master_in
+		# edges to and from loop,Main_in
 		
 	my @args = (track => $name,
 				endpoint => massaged_endpoint($track, $endpoint, $direction),
@@ -426,14 +426,17 @@ sub write_chains {
 
 	## write general options
 	
-	my $globals .= join " ", $config->{engine_globals}->{common},
-							join(',', '-G:jack',$this_engine->name,$this_engine->jack_transport_mode),
-							"-b",$config->buffersize,
-							$config->globals_realtime;
+	my @globals;
+	my $format = signal_format($config->{devices}->{jack}->{signal_format},2); # HARDCODED XXX
+	push @globals, $config->{ecasound_globals}->{common};
+	push @globals, "-f:$format", join(',', 	'-G:jack',
+										$config->{ecasound_jack_client_name},
+										$config->{jack_transport_mode}
+							) if $jack->{jackd_running};
+
+	push @globals, "-b", $config->buffersize, $config->globals_realtime;
+	my $globals = join " ", @globals;
 	
-	my $format = signal_format($config->{devices}->{jack}->{signal_format},2);
-	$globals .= " -f:$format" if $jack->{jackd_running};
-			
 	my $ecs_file = join "\n\n", 
 					"# ecasound chainsetup file",
 					"# general",
@@ -460,9 +463,9 @@ sub write_chains {
 sub setup_requires_realtime {
 	my $prof = $config->{realtime_profile};
 	if( $prof eq 'auto'){
-		grep{ ! $_->is_mix_track 
+		grep{ ! $_->is_mixing 
 				  and $_->is_user_track 
-				  and $_->rec_status =~ /REC|MON/
+				  and ($_->rec or $_->mon)
 			} Audio::Nama::audio_tracks() 
 	} elsif ( $prof eq 'realtime') {
 		my @fields = qw(soundcard jack_client jack_manual jack_ports_list);
@@ -511,11 +514,11 @@ its member tracks, and connecting them to its mix track.
 In the case of one track belonging to the Main (default) 
 bus, the initial graph would be:
 
-	soundcard_in -> sax -> Master -> soundcard_out
+	soundcard_in -> sax -> Main -> soundcard_out
 
 "soundcard_in" and "soundcard_out" will eventually be mapped
 to the appropriate JACK or ALSA source, depending on whether
-jackd is running. The Master track hosts the master fader,
+jackd is running. The Main track hosts the master fader,
 connects to the main output, and serves as the mix track for
 the Main bus.
 
@@ -535,7 +538,7 @@ the sax player) generates this additional route:
 Ecasound requires that we insert a loop device where signals fan 
 out or fan in.
 
-	soundcard_in -> sax -> sax_out -> Master -> soundcard_out
+	soundcard_in -> sax -> sax_out -> Main -> soundcard_out
 
 	                       sax_out -> soundcard_out
 
@@ -558,9 +561,9 @@ become the inputs and outputs of Ecasound chains.
 
 To create an Ecasound chain from 
 
-	Master -> soundcard_out 
+	Main -> soundcard_out 
 
-Nama uses 'Master' track attributes to provide
+Nama uses 'Main' track attributes to provide
 data. For example track index (1) serves as the chain_id,
 and the track's send settings determine the soundcard
 channel or other destination. 
